@@ -12,7 +12,7 @@ if not SPHINX_AUTODOC_RUNNING:
     import msvcrt
     from ctypes import windll
 
-from ctypes import pointer
+from ctypes import Array, pointer
 from ctypes.wintypes import DWORD, HANDLE
 from typing import (
     Callable,
@@ -252,6 +252,9 @@ class ConsoleInputReader:
         # Fill in 'data' for key presses.
         all_keys = [self._insert_key_data(key) for key in all_keys]
 
+        # Correct non-bmp characters that are passed as separate surrogate codes
+        all_keys = list(self._merge_paired_surrogates(all_keys))
+
         if self.recognize_paste and self._is_paste(all_keys):
             gen = iter(all_keys)
             k: Optional[KeyPress]
@@ -260,7 +263,7 @@ class ConsoleInputReader:
                 # Pasting: if the current key consists of text or \n, turn it
                 # into a BracketedPaste.
                 data = []
-                while k and (isinstance(k.key, str) or k.key == Keys.ControlJ):
+                while k and (not isinstance(k.key, Keys) or k.key == Keys.ControlJ):
                     data.append(k.data)
                     try:
                         k = next(gen)
@@ -289,7 +292,9 @@ class ConsoleInputReader:
 
         return KeyPress(key_press.key, data)
 
-    def _get_keys(self, read: DWORD, input_records) -> Iterator[KeyPress]:
+    def _get_keys(
+        self, read: DWORD, input_records: "Array[INPUT_RECORD]"
+    ) -> Iterator[KeyPress]:
         """
         Generator that yields `KeyPress` objects from the input records.
         """
@@ -313,6 +318,39 @@ class ConsoleInputReader:
                     for key_press in self._handle_mouse(ev):
                         yield key_press
 
+    @staticmethod
+    def _merge_paired_surrogates(key_presses: List[KeyPress]) -> Iterator[KeyPress]:
+        """
+        Combines consecutive KeyPresses with high and low surrogates into
+        single characters
+        """
+        buffered_high_surrogate = None
+        for key in key_presses:
+            is_text = not isinstance(key.key, Keys)
+            is_high_surrogate = is_text and "\uD800" <= key.key <= "\uDBFF"
+            is_low_surrogate = is_text and "\uDC00" <= key.key <= "\uDFFF"
+
+            if buffered_high_surrogate:
+                if is_low_surrogate:
+                    # convert high surrogate + low surrogate to single character
+                    fullchar = (
+                        (buffered_high_surrogate.key + key.key)
+                        .encode("utf-16-le", "surrogatepass")
+                        .decode("utf-16-le")
+                    )
+                    key = KeyPress(fullchar, fullchar)
+                else:
+                    yield buffered_high_surrogate
+                buffered_high_surrogate = None
+
+            if is_high_surrogate:
+                buffered_high_surrogate = key
+            else:
+                yield key
+
+        if buffered_high_surrogate:
+            yield buffered_high_surrogate
+
     @staticmethod
     def _is_paste(keys: List[KeyPress]) -> bool:
         """
@@ -328,7 +366,7 @@ class ConsoleInputReader:
         newline_count = 0
 
         for k in keys:
-            if isinstance(k.key, str):
+            if not isinstance(k.key, Keys):
                 text_count += 1
             if k.key == Keys.ControlM:
                 newline_count += 1
@@ -343,11 +381,13 @@ class ConsoleInputReader:
 
         result: Optional[KeyPress] = None
 
+        control_key_state = ev.ControlKeyState
         u_char = ev.uChar.UnicodeChar
-        ascii_char = u_char.encode("utf-8")
+        # Use surrogatepass because u_char may be an unmatched surrogate
+        ascii_char = u_char.encode("utf-8", "surrogatepass")
 
-        # NOTE: We don't use `ev.uChar.AsciiChar`. That appears to be latin-1
-        #       encoded. See also:
+        # NOTE: We don't use `ev.uChar.AsciiChar`. That appears to be the
+        # unicode code point truncated to 1 byte. See also:
         # https://github.com/ipython/ipython/issues/10004
         # https://github.com/jonathanslenders/python-prompt-toolkit/issues/389
 
@@ -367,10 +407,10 @@ class ConsoleInputReader:
         # First we handle Shift-Control-Arrow/Home/End (need to do this first)
         if (
             (
-                ev.ControlKeyState & self.LEFT_CTRL_PRESSED
-                or ev.ControlKeyState & self.RIGHT_CTRL_PRESSED
+                control_key_state & self.LEFT_CTRL_PRESSED
+                or control_key_state & self.RIGHT_CTRL_PRESSED
             )
-            and ev.ControlKeyState & self.SHIFT_PRESSED
+            and control_key_state & self.SHIFT_PRESSED
             and result
         ):
             mapping: Dict[str, str] = {
@@ -388,8 +428,8 @@ class ConsoleInputReader:
 
         # Correctly handle Control-Arrow/Home/End and Control-Insert/Delete keys.
         if (
-            ev.ControlKeyState & self.LEFT_CTRL_PRESSED
-            or ev.ControlKeyState & self.RIGHT_CTRL_PRESSED
+            control_key_state & self.LEFT_CTRL_PRESSED
+            or control_key_state & self.RIGHT_CTRL_PRESSED
         ) and result:
             mapping = {
                 Keys.Left: Keys.ControlLeft,
@@ -407,7 +447,7 @@ class ConsoleInputReader:
 
         # Turn 'Tab' into 'BackTab' when shift was pressed.
         # Also handle other shift-key combination
-        if ev.ControlKeyState & self.SHIFT_PRESSED and result:
+        if control_key_state & self.SHIFT_PRESSED and result:
             mapping = {
                 Keys.Tab: Keys.BackTab,
                 Keys.Left: Keys.ShiftLeft,
@@ -426,8 +466,8 @@ class ConsoleInputReader:
         # Turn 'Space' into 'ControlSpace' when control was pressed.
         if (
             (
-                ev.ControlKeyState & self.LEFT_CTRL_PRESSED
-                or ev.ControlKeyState & self.RIGHT_CTRL_PRESSED
+                control_key_state & self.LEFT_CTRL_PRESSED
+                or control_key_state & self.RIGHT_CTRL_PRESSED
             )
             and result
             and result.data == " "
@@ -438,8 +478,8 @@ class ConsoleInputReader:
         # detect this combination. But it's really practical on Windows.)
         if (
             (
-                ev.ControlKeyState & self.LEFT_CTRL_PRESSED
-                or ev.ControlKeyState & self.RIGHT_CTRL_PRESSED
+                control_key_state & self.LEFT_CTRL_PRESSED
+                or control_key_state & self.RIGHT_CTRL_PRESSED
             )
             and result
             and result.key == Keys.ControlJ
@@ -455,7 +495,7 @@ class ConsoleInputReader:
         #       all backslashes to be prefixed with escape. (Esc-\ has a
         #       meaning in E-macs, for instance.)
         if result:
-            meta_pressed = ev.ControlKeyState & self.LEFT_ALT_PRESSED
+            meta_pressed = control_key_state & self.LEFT_ALT_PRESSED
 
             if meta_pressed:
                 return [KeyPress(Keys.Escape, ""), result]
@@ -473,23 +513,26 @@ class ConsoleInputReader:
         MOUSE_MOVED = 0x0001
         MOUSE_WHEELED = 0x0004
 
+        event_flags = ev.EventFlags
+        button_state = ev.ButtonState
+
         result = []
         event_type: Optional[MouseEventType] = None
 
         # Move events.
-        if ev.EventFlags & MOUSE_MOVED:
-            if ev.ButtonState == FROM_LEFT_1ST_BUTTON_PRESSED:
+        if event_flags & MOUSE_MOVED:
+            if button_state == FROM_LEFT_1ST_BUTTON_PRESSED:
                 event_type = MouseEventType.MOUSE_DOWN_MOVE
 
         # Scroll events.
-        elif ev.EventFlags & MOUSE_WHEELED:
-            if ev.ButtonState > 0:
+        elif event_flags & MOUSE_WHEELED:
+            if button_state > 0:
                 event_type = MouseEventType.SCROLL_UP
             else:
                 event_type = MouseEventType.SCROLL_DOWN
 
         # Mouse down (left button).
-        elif ev.ButtonState == FROM_LEFT_1ST_BUTTON_PRESSED:
+        elif button_state == FROM_LEFT_1ST_BUTTON_PRESSED:
             event_type = MouseEventType.MOUSE_DOWN
 
         # No key pressed anymore: mouse up.
@@ -593,7 +636,9 @@ class _Win32Handles:
 
 
 @contextmanager
-def attach_win32_input(input: _Win32InputBase, callback: Callable[[], None]):
+def attach_win32_input(
+    input: _Win32InputBase, callback: Callable[[], None]
+) -> Iterator[None]:
     """
     Context manager that makes this input active in the current event loop.
 
@@ -620,7 +665,7 @@ def attach_win32_input(input: _Win32InputBase, callback: Callable[[], None]):
 
 
 @contextmanager
-def detach_win32_input(input: _Win32InputBase):
+def detach_win32_input(input: _Win32InputBase) -> Iterator[None]:
     win32_handles = input.win32_handles
     handle = input.handle
 
